export module Garden;

#include <deque> // C++ 23 is required to import the standard library as module...

export import Vegetables; // "Vegetables" will be (also) imported for importers of 'Garden'...

/* 'std::derived_from' is about *PUBLIC* inheritance */
export template<typename Specific_vegetable> requires std::derived_from<Specific_vegetable, Vegetable> class Garden {
	// Alternative syntax: 'export template<typename Specific_vegetable> class Garden requires std::derived_from<Specific_vegetable, Vegetable> {'
private:
	std::deque<Specific_vegetable> _vegetables;
public:
	const std::deque<Specific_vegetable>& vegetables() const { return _vegetables; } // Getter...

	bool is_bigger_than_V1(const Garden<Specific_vegetable>& garden) const { return this->_vegetables.size() > garden._vegetables.size(); }

	template <typename V> bool is_bigger_than_V2(const Garden<V>& garden) const
		requires std::derived_from<V, Specific_vegetable> {
		return this->_vegetables.size() > garden.vegetables().size();
	}

	template <typename V> bool is_bigger_than_V3(const Garden<V>& garden) const
		requires std::derived_from<V, Specific_vegetable> || std::derived_from<Specific_vegetable, V> {
		return this->_vegetables.size() > garden.vegetables().size();
	}

	void dig_out() { this->_vegetables.pop_back(); }
	void plant(const Specific_vegetable& specific_vegetable) { this->_vegetables.push_front(specific_vegetable); }
	int size() const { return this->_vegetables.size(); }

	Garden& operator=(Garden&& garden) noexcept { // Move assignment
		this->_vegetables.clear();
		while (!garden._vegetables.empty()) {
			this->plant(garden._vegetables.back());
			garden.dig_out();
		}
	}

	template <typename V> void transfer(Garden<V>& garden)
		requires std::derived_from<V, Specific_vegetable> {
		this->_vegetables.clear();
		while (!garden.vegetables().empty()) {
			this->plant(garden.vegetables().back());
			garden.dig_out();
		}
	}
};
