// Must be placed first:
module; // Vegetables.ixx (".ixx" is the default (required in Visual Studio!) suffix for module interface in Visual Studio)

export module Vegetables;

#include <concepts>
#include <type_traits> // 'std::is_base_of_v'

export enum Color : unsigned short { Green = 0x00FF00, Orange = 0xFFA500, Purple = 0xA020F0, Red = 0xFF0000/* Etc. */ }; // Scoped enumeration...

export class Vegetable {
public:
	virtual Color getColor() = 0;
};

export class Salad : public Vegetable {
public:
	inline Color getColor() { return Color::Green; }
};

export class Batavia : public Salad {};

export class Lettuce : public Salad {};

// Not exported:
class Carot : public Vegetable {
public:
	inline Color getColor() { return Color::Orange; }
};
class Cosmic_carot : public Carot {
public:
	inline Color getColor() { return Color::Purple; }
};

template <typename T> concept Is_carot = std::is_base_of_v<Carot, T>; // <=> 'std::is_base_of<Carot, T>::value'
// At most 2 kinds of carots in the garden:
template<Is_carot... Carots> requires ((sizeof... (Carots) > 0 && sizeof... (Carots) <= 2)) class Carot_garden { /* ... */ };
template<typename Key_ingredient, typename... Ingredients> requires Is_carot<Key_ingredient> class Carot_cake {
	void cook(Key_ingredient carot, Ingredients... ingredients) { /* ... */ }
};
class Cosmic_carot_cake : public Carot_cake<Cosmic_carot /*, ... */> {};